package net.sourceforge.rapidprops;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.junit.Test;
import static org.junit.Assert.*;

public class PropsBuilderTest {

	public static interface TestProps1 {
		String getRecipient();
		int getAge();

		@Prop(key = "first.name")
		String getFirstName();

		@Prop(key = "last.name", defaultValue = "Smith")
		String getLastName();

		@Prop(key = "some.byte")
		byte getSomePrimitiveByte();

		@Prop(key = "some.short")
		short getSomePrimitiveShort();

		@Prop(key = "some.int")
		int getSomePrimitiveInt();

		@Prop(key = "some.long")
		long getSomePrimitiveLong();

		@Prop(key = "some.double")
		double getSomePrimitiveDouble();

		@Prop(key = "some.float")
		float getSomePrimitiveFloat();

		@Prop(key = "some.boolean")
		boolean getSomePrimitiveBoolean();

		@Prop(key = "some.byte")
		Byte getSomeByte();

		@Prop(defaultValue = Prop.NULL)
		Byte getSomeOtherByte();

		@Prop(key = "some.short")
		Short getSomeShort();

		@Prop(defaultValue = Prop.NULL)
		Short getSomeOtherShort();

		@Prop(key = "some.int")
		Integer getSomeInteger();

		@Prop(defaultValue = Prop.NULL)
		Integer getSomeOtherInteger();

		@Prop(key = "some.long")
		Long getSomeLong();

		@Prop(defaultValue = Prop.NULL)
		Long getSomeOtherLong();

		@Prop(key = "some.double")
		Double getSomeDouble();

		@Prop(defaultValue = Prop.NULL)
		Double getSomeOtherDouble();

		@Prop(key = "some.float")
		Float getSomeFloat();

		@Prop(defaultValue = Prop.NULL)
		Float getSomeOtherFloat();

		@Prop(key = "some.boolean")
		Boolean getSomeBoolean();

		@Prop(defaultValue = Prop.NULL)
		Boolean getSomeOtherBoolean();

		@Prop(key = "some.number.array")
		byte[] getSomePrimitiveByteArray();

		@Prop(key = "some.number.array")
		Byte[] getSomeByteArray();

		@Prop(key = "some.number.array")
		short[] getSomePrimitiveShortArray();

		@Prop(key = "some.number.array")
		Short[] getSomeShortArray();

		@Prop(key = "some.number.array")
		int[] getSomePrimitiveIntArray();

		@Prop(key = "some.number.array")
		Integer[] getSomeIntegerArray();

		@Prop(key = "some.number.array")
		long[] getSomePrimitiveLongArray();

		@Prop(key = "some.number.array")
		Long[] getSomeLongArray();

		@Prop(key = "some.number.array")
		double[] getSomePrimitiveDoubleArray();

		@Prop(key = "some.number.array")
		Double[] getSomeDoubleArray();

		@Prop(key = "some.number.array")
		float[] getSomePrimitiveFloatArray();

		@Prop(key = "some.number.array")
		Float[] getSomeFloatArray();

		@Prop(key = "some.boolean.array")
		boolean[] getSomePrimitiveBooleanArray();

		@Prop(key = "some.boolean.array")
		Boolean[] getSomeBooleanArray();

		@Prop(key = "some.string.array")
		String[] getSomeStringArray();

		@Prop(key = "some.string.array", nullAlias="nobody")
		String[] getSomeStringArrayNobodyMeansNull();

		@Prop(key = "some.string.array", nullAlias="")
		String[] getSomeStringArrayEmptyMeansNull();

		@Prop(key = "some.string.array", nullAlias=PropConstants.DEFAULT_DELIMITER)
		String[] getSomeStringArrayNoNull();

		@Prop(key = "semicolon.delimited.array", delimiter=";", defaultElementValue="-5")
		int[] getSemicolonDelimitedPrimitiveIntArray();

		@Prop(key = "some.number.array")
		List<Double> getSomeDoubleList();

		@Prop(key = "some.string.array")
		List<String> getSomeStringList();

		@Prop(key = "some.incomplete.array", defaultElementValue="-1")
		int[] getSomeIncompletePrimitiveIntArray();

		@Prop(key = "some.number.indexed.map.of.string.array")
		Map<Float, String[]> getSomeFloatIndexedMapOfStringArray();
	}

	@Test
	public void test1() throws IOException {
		TestProps1 props = PropsBuilder.createFromFile(TestProps1.class, "src/test/resources/test.properties");

		assertEquals("world", props.getRecipient());
		assertEquals(33, props.getAge());
		assertEquals("John", props.getFirstName());
		assertEquals("Doe", props.getLastName());

		assertEquals((byte)127, props.getSomePrimitiveByte());
		assertEquals((short)12345, props.getSomePrimitiveShort());
		assertEquals(1234567890, props.getSomePrimitiveInt());
		assertEquals(1234567654321L, props.getSomePrimitiveLong());
		assertEquals(1234567.7654321, props.getSomePrimitiveDouble(), 0.001);
		assertEquals(123.321f, props.getSomePrimitiveFloat(), 0.001);
		assertEquals(true, props.getSomePrimitiveBoolean());

		assertEquals(new Byte((byte)127), props.getSomeByte());
		assertEquals(null, props.getSomeOtherByte());

		assertEquals(new Short((short)12345), props.getSomeShort());
		assertEquals(null, props.getSomeOtherShort());

		assertEquals(new Integer(1234567890), props.getSomeInteger());
		assertEquals(null, props.getSomeOtherInteger());

		assertEquals(new Long(1234567654321L), props.getSomeLong());
		assertEquals(null, props.getSomeOtherLong());

		assertEquals(new Double(1234567.7654321), props.getSomeDouble());
		assertEquals(null, props.getSomeOtherDouble());

		assertEquals(new Float(123.321f), props.getSomeFloat());
		assertEquals(null, props.getSomeOtherFloat());

		assertEquals(Boolean.TRUE, props.getSomeBoolean());
		assertEquals(null, props.getSomeOtherBoolean());

		assertArrayEquals(new byte[] {3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23}, props.getSomePrimitiveByteArray());
		assertArrayEquals(new Byte[] {3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23}, props.getSomeByteArray());
		assertArrayEquals(new short[] {3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23}, props.getSomePrimitiveShortArray());
		assertArrayEquals(new Short[] {3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23}, props.getSomeShortArray());
		assertArrayEquals(new int[] {3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23}, props.getSomePrimitiveIntArray());
		assertArrayEquals(new Integer[] {3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23}, props.getSomeIntegerArray());
		assertArrayEquals(new long[] {3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23}, props.getSomePrimitiveLongArray());
		assertArrayEquals(new Long[] {3L, 5L, 7L, 9L, 11L, 13L, 15L, 17L, 19L, 21L, 23L}, props.getSomeLongArray());
		assertArrayEquals(new double[] {3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23}, props.getSomePrimitiveDoubleArray(), 0.001);
		assertArrayEquals(new Double[] {3d, 5d, 7d, 9d, 11d, 13d, 15d, 17d, 19d, 21d, 23d}, props.getSomeDoubleArray());
		assertArrayEquals(new float[] {3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23}, props.getSomePrimitiveFloatArray(), 0.001f);
		assertArrayEquals(new Float[] {3f, 5f, 7f, 9f, 11f, 13f, 15f, 17f, 19f, 21f, 23f}, props.getSomeFloatArray());

		assertTrue(Arrays.equals(new boolean[] {true, true, false, true, false, false, true}, props.getSomePrimitiveBooleanArray()));
		assertArrayEquals(new Boolean[] {true, true, false, true, false, false, true}, props.getSomeBooleanArray());

		assertArrayEquals(new String[] {"John", "", "Mary", null, "Bill", "nobody", "Kate"}, props.getSomeStringArray());
		assertArrayEquals(new String[] {"John", "", "Mary", "null", "Bill", null, "Kate"}, props.getSomeStringArrayNobodyMeansNull());
		assertArrayEquals(new String[] {"John", null, "Mary", "null", "Bill", "nobody", "Kate"}, props.getSomeStringArrayEmptyMeansNull());
		assertArrayEquals(new String[] {"John", "", "Mary", "null", "Bill", "nobody", "Kate"}, props.getSomeStringArrayNoNull());

		assertArrayEquals(new int[] {2, 4, 6, 8, 10, -5, -5}, props.getSemicolonDelimitedPrimitiveIntArray());

		assertEquals(Arrays.asList(new Double[] {3d, 5d, 7d, 9d, 11d, 13d, 15d, 17d, 19d, 21d, 23d}), props.getSomeDoubleList());
		assertEquals(Arrays.asList(new String[] {"John", "", "Mary", null, "Bill", "nobody", "Kate"}), props.getSomeStringList());

		assertArrayEquals(new int[] {1, -1, -1, -1, -1, 2, -1, 3, 4, -1, -1}, props.getSomeIncompletePrimitiveIntArray());

		Map<Float, String[]> someFloatIndexedMapOfStringArray = props.getSomeFloatIndexedMapOfStringArray();
		assertArrayEquals(new String[] {"twenty", "zwanzig", "vingt", "veinte"}, someFloatIndexedMapOfStringArray.get(20f));
	}
}
